<?php
// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;
/**
 * Main Class
 */
if( !class_exists( 'WP_My_Instagram_Main') ):
	
	class WP_My_Instagram_Main{

		/**
		 * Display Feed
		 * @since 1.0.0
		 */
		public function images_only( $media_item ) {

			if ( $media_item['type'] == 'image' )
				return true;

			return false;
		}

		/**
		 * Display Feed
		 * @since 1.0.0
		 * @version 1.1.0
		 */
		static function display_feed( $args = array() ){

			$args = wp_parse_args( $args, array(
	 			'username' 	=> '',
	 			'hashtag' 	=> '', //If hashtag entered, use hashtag
				'limit' 	=> 6,
				'size'		=> 'large', //thumbnail, small, large, original
				'layout'	=> '3', // default, 2, 3, 4, 5, 6, 8, 10
				'target'	=> '_self',
				'link'		=> ''
			) );

			$args['layout'] = !in_array( $args['layout'], array( 'default', '2', '3', '4', '5', '6', '8', '10' ) ) ? 'default' : $args['layout'];
			$args['username'] = (string) $args['username'];
			$args['hashtag'] = (string) $args['hashtag'];
			$scrape_key = !empty( $args['hashtag'] ) ? $args['hashtag'] : $args['username'];
			$use_hashtag = !empty( $args['hashtag'] ) ? true : false;
			$prefix = $use_hashtag ? 'tag' : 'user';

			if( empty( $args['username'] ) && empty( $args['hashtag'] ) )
				echo esc_html__( 'You need to input your username or hashtag.', 'wp-my-instagram' );

			// Test Drive
			// $a = self::_get_batch('https://www.instagram.com/jscorsechen/', false);
			// $a = self::_get_batch('https://www.instagram.com/explore/tags/snkr/', true);
			
			$output = '';
			$cached = false;
			if( self::get_cache( $scrape_key, (int) $args['limit'], $use_hashtag ) ){

				$media_array = self::scrape_instagram( $scrape_key, (int) $args['limit'], $use_hashtag );
				$cached = true;

			}else{
				$media_array = array();
			}

			if ( is_wp_error( $media_array ) ) {

				echo wp_kses_post( $media_array->get_error_message() );

			} else {

				// filter for images only?
				if ( $images_only = apply_filters( 'wpmi_images_only', false ) ) {
					$media_array = array_filter( $media_array, array( __CLASS__, 'images_only' ) );
				}
				// slice list down to required limit
				$media_array = array_slice( $media_array, 0, (int) $args['limit'] );
				
				if( !$cached ){
					wp_enqueue_script( 'wp-my-instagram' );
				}
				?>
				<div id="wpmi-<?php echo esc_attr( uniqid() );?>" class="wp-my-instagram wpmi" data-args="<?php echo esc_attr( json_encode($args) );?>" data-cached="<?php echo esc_attr( $cached ? 'true' : 'false' );?>">
					<ul class="wpmi-list wpmi-layout-<?php echo esc_attr( $args['layout'] );?>">
						<?php
							echo self::make_items( $media_array, $args );
						?>
					</ul>
					<?php 

					if( $args['link'] ){
						$url_instagram = '//instagram.com/';
						$url = $use_hashtag ? $url_instagram . 'explore/tags/' . $args['hashtag'] : $url_instagram . $args['username'];
						$rel = $use_hashtag ? '' : ' rel="me"';
					?>
					<div class="wpmi-me"><a href="<?php echo esc_url( trailingslashit( $url ) ); ?>" target="<?php echo esc_attr( $args['target'] ); ?>"<?php echo !empty( $rel) ? $rel : '';?>><?php echo wp_kses_post( $args['link'] ); ?></a></div>
					<?php
					}?>
				</div>
				<?php
			}
		}
		/**
		 * Make items
		 * @since 1.0.2
		 */
		static function make_items( $media_array = array(), $args = array() ){

			$output = '';
			$template_part = self::get_template();

			foreach ( $media_array as $item ) {
				// copy the else line into a new file (wp-my-instagram/loop.php) within your theme and customise accordingly
				$output .= strtr( $template_part, array(
					'{{link}}'		=> esc_url( $item['link'] ),
					'{{target}}'	=> esc_attr( $item['link'] ),
					'{{image}}'			=> esc_url( $item[$args['size']] ),
					'{{description}}'	=> esc_attr( $item['description'] ),
				) );
			}

			return $output;
		}
		/**
		 * Get item Template
		 * @since 1.0.2
		 */
		static function get_template(){
			$template_part_default = '<li class="wpmi-item"><a href="{{link}}" target="{{target}}"><img src="{{image}}"  alt="" title="{{description}}"/></a></li>';
			$template_part = apply_filters( 'wpmi_item_template', $template_part_default );
			if( empty( $template_part )) {
				$template_part = $template_part_default;
			}

			return $template_part;
		}
        /**
		 * Get cache name
		 * @var $key 	string username or hashtag
		 * @var $limit 	int
		 * @var $prefix 	string
		 * @since 1.0
		 * @version 1.1.0
		 */
		static function get_cache_name( $key, $limit, $use_hashtag = false ){
			$prefix = $use_hashtag ? 'tag' : 'user';
			$key = strtolower( $key );
			$key = strtr( $key, array( '@' => '', '#' => '' ) );
			$count = ceil( $limit/12 );
			return 'wpmi-' . sanitize_title_with_dashes( $prefix . '-' . $key . '-' . $count );
		}
		/**
		 * Get cache
		 * @since 1.0.2
		 * @version 1.1.0
		 */
		static function get_cache( $username, $limit, $use_hashtag = false){

			return get_transient( self::get_cache_name( $username, $limit, $use_hashtag ) );
		}
		/**
		 * Delete Cache
		 * @since 1.0
		 * @version 1.1.0
		 */
		static function delete_cache( $username, $limit, $use_hashtag = false ){

			$transient_name = self::get_cache_name( $username, $limit, $use_hashtag );

			delete_transient( $transient_name );

		}
		/**
		 * Get Batch
		 * @since 1.0
		 * @version 1.1.0
		 */
		private static function _get_batch( $url, $use_hashtag = false ){

			if( empty( $url ) )
				return false;

			$remote = wp_remote_get( $url, array( 'timeout' => 120, 'sslverify' => false ) );

			if ( is_wp_error( $remote ) )
				return new WP_Error( 'site_down', esc_html__( 'Unable to communicate with Instagram.', 'wp-my-instagram' ) );

			if ( 200 != wp_remote_retrieve_response_code( $remote ) )
				return new WP_Error( 'invalid_response', esc_html__( 'Instagram did not return a 200.', 'wp-my-instagram' ) );

			$shared_data = explode( 'window._sharedData = ', $remote['body'] );
			$insta_json = explode( ';</script>', $shared_data[1] );
			$insta_array = json_decode( $insta_json[0], TRUE );

			if ( ! $insta_array )
				return new WP_Error( 'bad_json', esc_html__( 'Instagram has returned invalid data.', 'wp-my-instagram' ) );

			// print_r( $insta_array['entry_data']['TagPage'][0]['graphql']['hashtag']['edge_hashtag_to_media']['edges'] ); die();

			// old
			// Username
			if ( !$use_hashtag && isset( $insta_array['entry_data']['ProfilePage'][0]['user']['media']['nodes'] ) ) {
				$images = $insta_array['entry_data']['ProfilePage'][0]['user']['media']['nodes'];
			}
			// Hashtag
			// Old node: $insta_array['entry_data']['TagPage'][0]['tag']['media']['nodes']
			// New node 2018: $insta_array['entry_data']['TagPage'][0]['graphql']['hashtag']['edge_hashtag_to_media']['edges']

			elseif( $use_hashtag && isset( $insta_array['entry_data']['TagPage'][0]['graphql']['hashtag']['edge_hashtag_to_media']['edges'] ) ){
				$images = $insta_array['entry_data']['TagPage'][0]['graphql']['hashtag']['edge_hashtag_to_media']['edges'];
			}else {
				return new WP_Error( 'bad_json_2', esc_html__( 'Instagram has returned invalid data.', 'wp-my-instagram' ) );
			}

			if ( ! is_array( $images ) )
				return new WP_Error( 'bad_array', esc_html__( 'Instagram has returned invalid data.', 'wp-my-instagram' ) );

			$instagram = array();
			// print_r( $images );
			foreach ( $images as $image ) {

				if( $use_hashtag ){
					$image = $image['node'];
				}
				
				$image['thumbnail_src'] = preg_replace( '/^https?\:/i', '', $image['thumbnail_src'] );

				$display_key = $use_hashtag ? 'display_url' : 'display_src';
				$original = preg_replace( '/^https?\:/i', '', $image[$display_key] );
				$img_code = $use_hashtag ? $image['shortcode'] : $image['code'];

				// handle both types of CDN url
				if ( ( strpos( $image['thumbnail_src'], 's640x640' ) !== false ) ) {
					$image['thumbnail'] = str_replace( 's640x640', 's160x160', $image['thumbnail_src'] );
					$image['small'] = str_replace( 's640x640', 's320x320', $image['thumbnail_src'] );
				} else {
					$urlparts = wp_parse_url( $image['thumbnail_src'] );
					$pathparts = explode( '/', $urlparts['path'] );
					array_splice( $pathparts, 3, 0, array( 's160x160' ) );
					$image['thumbnail'] = '//' . $urlparts['host'] . implode( '/', $pathparts );
					$pathparts[3] = 's320x320';
					$image['small'] = '//' . $urlparts['host'] . implode( '/', $pathparts );
				}

				$image['large'] = $image['thumbnail_src'];

				if ( $image['is_video'] == true ) {
					$type = 'video';
				} else {
					$type = 'image';
				}

				$caption = __( 'Instagram Image', 'wp-my-instagram' );
				if ( ! empty( $image['caption'] ) ) {
					$caption = $image['caption'];
				}

				$instagram[] = array(
					'id'			=> $image['id'],
					'description'   => $caption,
					'link'		  	=> trailingslashit( '//instagram.com/p/' . $img_code ),
					'time'		  	=> $image['date'],
					'comments'	  	=> $image['comments']['count'],
					'likes'		 	=> $image['likes']['count'],
					'thumbnail'	 	=> $image['thumbnail'],
					'small'			=> $image['small'],
					'large'			=> $image['large'],
					'original'		=> $original,
					'type'		  	=> $type
				);
				// print_r( $instagram );
			}

			return $instagram;
		}	
		/**
		 * Get Instagram
		 * 
		 * @var 
		 * @since 1.0
		 * @version 1.1.0
		 * Special Thanks to Scott Evans - wp-instagram-widget
		 */
		// based on https://gist.github.com/cosmocatalano/4544576

		static function scrape_instagram( $_scrape_key, $_limit = 12, $_use_hashtag = false ) {

			$base_url = 'https://instagram.com/';
			$_scrape_key = strtolower( $_scrape_key );
			$_scrape_key = str_replace( '@', '', $_scrape_key );
			$_scrape_key = trim( $_scrape_key );

			$url = $base_url . $_scrape_key;
			// If use hashtag
			if( $_use_hashtag ){
				$url = $base_url . 'explore/tags/' . $_scrape_key;
			}

			// Get cache
			$cache_name = self::get_cache_name( $_scrape_key, $_limit, $_use_hashtag );
			$cache_time = apply_filters( 'wpmi_cache_time', HOUR_IN_SECONDS*2 );

			if ( false === ( $feed = get_transient( $cache_name ) ) ) {
				$feed = array();	
				$batches = ceil( $_limit/12 );
				$last_id = false;

				for ( $i=0; $i < $batches; $i++ ) {

					// Trick facebook pagination :P
					$batch = $last_id ? $url . "/?max_id={$last_id}" : $url;

					$instagram = self::_get_batch( $batch, $_use_hashtag );

					if( is_array( $instagram ) ){
						$last_image = end( $instagram );
						$last_id = !empty( $last_image['id'] ) ? $last_image['id'] : 0;
						$feed = array_merge( $feed, $instagram );
					}
				}

				// do not set an empty transient - should help catch private or empty accounts
				if ( ! empty( $feed ) ) {
					$feed = base64_encode( serialize( $feed ) );
					set_transient( $cache_name, $feed, $cache_time );
				}
			}

			if ( ! empty( $feed ) ) {

				return unserialize( base64_decode( $feed ) );

			} else {

				return new WP_Error( 'no_images', esc_html__( 'Instagram did not return any images.', 'wp-my-instagram' ) );

			}
		}
	}
endif;

if( !function_exists( 'wp_my_instagram') ){
	function wp_my_instagram( $args = array()){

		if( !is_array( $args ) ){
			esc_html_e( 'This param must be array.', 'wp-my-instagram' );
			return;
		}

		WP_My_Instagram_Main::display_feed( $args );

	}
}